==> llvm-toolchain-3.7_3.7.1.orig-lldb.tar.bz2: lldb_3.7.1~+final/source/Plugins/ABI/SysV-mips64/ABISysV_mips64.h <==
//===-- ABISysV_mips64.h ----------------------------------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef liblldb_ABISysV_mips64_h_
#define liblldb_ABISysV_mips64_h_

// C Includes
// C++ Includes
// Other libraries and framework includes
// Project includes
#include "lldb/lldb-private.h"
#include "lldb/Target/ABI.h"

class ABISysV_mips64 :
    public lldb_private::ABI
{
public:

    ~ABISysV_mips64()
    {
    }

    virtual size_t
    GetRedZoneSize () const;

    virtual bool
    PrepareTrivialCall (lldb_private::Thread &thread, 
                        lldb::addr_t sp,
                        lldb::addr_t functionAddress,
                        lldb::addr_t returnAddress, 
                        llvm::ArrayRef<lldb::addr_t> args) const;

    virtual bool
    GetArgumentValues (lldb_private::Thread &thread,
                       lldb_private::ValueList &values) const;

    virtual lldb_private::Error
    SetReturnValueObject(lldb::StackFrameSP &frame_sp, lldb::ValueObjectSP &new_value);

protected:
    lldb::ValueObjectSP
    GetReturnValueObjectSimple (lldb_private::Thread &thread,
                                lldb_private::ClangASTType &ast_type) const;

public:    
    virtual lldb::ValueObjectSP
    GetReturnValueObjectImpl (lldb_private::Thread &thread,
                          lldb_private::ClangASTType &type) const;

    virtual bool
    CreateFunctionEntryUnwindPlan (lldb_private::UnwindPlan &unwind_plan);
    
    virtual bool
    CreateDefaultUnwindPlan (lldb_private::UnwindPlan &unwind_plan);

    virtual bool
    RegisterIsVolatile (const lldb_private::RegisterInfo *reg_info);

    // The SysV mips ABI requires that stack frames be 16 byte aligned.
    // When there is a trap handler on the stack, e.g. _sigtramp in userland
    // code, we've seen that the stack pointer is often not aligned properly
    // before the handler is invoked.  This means that lldb will stop the unwind
    // early -- before the function which caused the trap.
    //
    // To work around this, we relax that alignment to be just word-size (8-bytes).
    // Whitelisting the trap handlers for user space would be easy (_sigtramp) but
    // in other environments there can be a large number of different functions
    // involved in async traps.
    virtual bool
    CallFrameAddressIsValid (lldb::addr_t cfa)
    {
        // Make sure the stack call frame addresses are 8 byte aligned
        if (cfa & (8ull - 1ull))
            return false;   // Not 8 byte aligned
        if (cfa == 0)
            return false;   // Zero is not a valid stack address
        return true;
    }

    virtual bool
    CodeAddressIsValid (lldb::addr_t pc)
    {
       if (pc & (4ull - 1ull))
           return false;   // Not 4 byte aligned
        
        // Anything else if fair game..
        return true;
    }

    virtual const lldb_private::RegisterInfo *
    GetRegisterInfoArray (uint32_t &count);
    //------------------------------------------------------------------
    // Static Functions
    //------------------------------------------------------------------
    static void
    Initialize();

    static void
    Terminate();

    static lldb::ABISP
    CreateInstance (const lldb_private::ArchSpec &arch);

    static lldb_private::ConstString
    GetPluginNameStatic();

    //------------------------------------------------------------------
    // PluginInterface protocol
    //------------------------------------------------------------------
    virtual lldb_private::ConstString
    GetPluginName();

    virtual uint32_t
    GetPluginVersion();

protected:
    void
    CreateRegisterMapIfNeeded ();

    bool
    RegisterIsCalleeSaved (const lldb_private::RegisterInfo *reg_info);

private:
    ABISysV_mips64() : lldb_private::ABI() { } // Call CreateInstance instead.
};

#endif  // liblldb_ABI_h_

==> llvm-toolchain-3.8_3.8.orig-lldb.tar.bz2: lldb_3.8~+final/source/Plugins/ABI/SysV-mips64/ABISysV_mips64.h <==
//===-- ABISysV_mips64.h ----------------------------------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef liblldb_ABISysV_mips64_h_
#define liblldb_ABISysV_mips64_h_

// C Includes
// C++ Includes
// Other libraries and framework includes
// Project includes
#include "lldb/lldb-private.h"
#include "lldb/Target/ABI.h"

class ABISysV_mips64 :
    public lldb_private::ABI
{
public:
    ~ABISysV_mips64() override = default;

    size_t
    GetRedZoneSize() const override;

    bool
    PrepareTrivialCall(lldb_private::Thread &thread,
                       lldb::addr_t sp,
                       lldb::addr_t functionAddress,
                       lldb::addr_t returnAddress,
                       llvm::ArrayRef<lldb::addr_t> args) const override;

    bool
    GetArgumentValues(lldb_private::Thread &thread,
                      lldb_private::ValueList &values) const override;

    lldb_private::Error
    SetReturnValueObject(lldb::StackFrameSP &frame_sp, lldb::ValueObjectSP &new_value) override;

    lldb::ValueObjectSP
    GetReturnValueObjectImpl(lldb_private::Thread &thread,
                             lldb_private::CompilerType &type) const override;

    bool
    CreateFunctionEntryUnwindPlan(lldb_private::UnwindPlan &unwind_plan) override;
    
    bool
    CreateDefaultUnwindPlan(lldb_private::UnwindPlan &unwind_plan) override;

    bool
    RegisterIsVolatile(const lldb_private::RegisterInfo *reg_info) override;

    // The SysV mips ABI requires that stack frames be 16 byte aligned.
    // When there is a trap handler on the stack, e.g. _sigtramp in userland
    // code, we've seen that the stack pointer is often not aligned properly
    // before the handler is invoked.  This means that lldb will stop the unwind
    // early -- before the function which caused the trap.
    //
    // To work around this, we relax that alignment to be just word-size (8-bytes).
    // Whitelisting the trap handlers for user space would be easy (_sigtramp) but
    // in other environments there can be a large number of different functions
    // involved in async traps.
    bool
    CallFrameAddressIsValid(lldb::addr_t cfa) override
    {
        // Make sure the stack call frame addresses are 8 byte aligned
        if (cfa & (8ull - 1ull))
            return false;   // Not 8 byte aligned
        if (cfa == 0)
            return false;   // Zero is not a valid stack address
        return true;
    }

    bool
    CodeAddressIsValid(lldb::addr_t pc) override
    {
       if (pc & (4ull - 1ull))
           return false;   // Not 4 byte aligned
        
        // Anything else if fair game..
        return true;
    }

    const lldb_private::RegisterInfo *
    GetRegisterInfoArray(uint32_t &count) override;

    //------------------------------------------------------------------
    // Static Functions
    //------------------------------------------------------------------

    static void
    Initialize();

    static void
    Terminate();

    static lldb::ABISP
    CreateInstance (const lldb_private::ArchSpec &arch);

    static lldb_private::ConstString
    GetPluginNameStatic();

    //------------------------------------------------------------------
    // PluginInterface protocol
    //------------------------------------------------------------------

    lldb_private::ConstString
    GetPluginName() override;

    uint32_t
    GetPluginVersion() override;

protected:
    void
    CreateRegisterMapIfNeeded ();

    lldb::ValueObjectSP
    GetReturnValueObjectSimple(lldb_private::Thread &thread,
                               lldb_private::CompilerType &ast_type) const;

    bool
    RegisterIsCalleeSaved (const lldb_private::RegisterInfo *reg_info);

private:
    ABISysV_mips64() : 
        lldb_private::ABI() 
    {
         // Call CreateInstance instead.
    }
};

#endif // liblldb_ABISysV_mips64_h_
